--
-- VarioSpreadingWidth
-- specialization for new working width of spreaders (Farming Simulator 17)
-- 
--
--	@author:			gotchTOM
--	@date: 				19-Mar-2017
--	@version: 		v0.05
--  @changes:			

VarioSpreadingWidth = {};

function VarioSpreadingWidth.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Sprayer, specializations);
end;

function VarioSpreadingWidth:load(savegame)
	
	local workAreas;
	for _,workArea in pairs(self.workAreaByType) do
		for _,a in pairs(workArea) do
			local areaTypeStr = WorkArea.areaTypeIntToName[a.type];
			if areaTypeStr == "sprayer" then
				workAreas = self.workAreaByType[a.type];
			end;
		end;
	end;
	local isFertilizerSpreader = false;
	local isManureSpreader = false;
	for fillUnit,_ in pairs(self.fillUnits) do
		local fillTypes = self:getUnitFillTypes(fillUnit);
		for fillType,k in pairs(fillTypes) do
			if fillType == FillUtil.FILLTYPE_FERTILIZER then
				isFertilizerSpreader = k;
				self.isSpreader = k;
			end;
			if fillType == FillUtil.FILLTYPE_MANURE then
				isManureSpreader = k;
				self.isSpreader = k;
			end;
		end;
	end;
	if self.isSpreader then
		if self.sprayUsageScale ~= nil then
			local workingWidth = math.floor(self.sprayUsageScale.workingWidth*10+.5)/10;
			self.varioSpreadingWidth = {};
			self.varioSpreadingWidth.workingWidth = workingWidth;
			self.varioSpreadingWidth.workingWidthBackup = workingWidth;
			self.varioSpreadingWidth.buttonIsPressed = false;
			self.varioSpreadingWidth.buttonTimer = 0;
			if isFertilizerSpreader then
				self.varioSpreadingWidth.maxWorkWidth = 50;
				self.varioSpreadingWidth.minWorkWidth = 12;
			elseif isManureSpreader then
				if workingWidth >= 12 and workingWidth <= 24 then
					self.varioSpreadingWidth.maxWorkWidth = 24;
					self.varioSpreadingWidth.minWorkWidth = 12;
				else
					self.isSpreader = false;
					print("manure spreader working width < 12m or > 24m -> VarioSpreadingWidth can not be used!")
					self.varioSpreadingWidth = nil;
					return;
				end;
			end;
			self.orgWorkAreas = {};
			for k,v in pairs(self.workAreas) do
				local area = {};
				area.workWidth = workingWidth;
				local xW,yW,zW = getTranslation(v.width)
				area.width = {x=xW,y=yW,z=zW};
				local xH,yH,zH = getTranslation(v.height)
				area.height = {x=xH,y=yH,z=zH};
				table.insert(self.orgWorkAreas,area)
				local i3dWidth = math.abs(math.floor(area.width.x+.5)) + math.abs(math.floor(area.height.x+.5));
				local scaleWidth = math.floor(workingWidth+.5)
				if i3dWidth ~= scaleWidth then
					self.isSpreader = false;
					print("sprayUsageScale.workingWidth and workingWidth in the i3d-file are not the same -> VarioSpreadingWidth can not be used!")
					self.varioSpreadingWidth = nil;
					self.orgWorkAreas = nil;
					return;
				end;			
			end;
			local x,y,z = getTranslation(self.aiLeftMarker)
			self.orgAiLeft = {x,y,z};
			x,y,z = getTranslation(self.aiRightMarker)
			self.orgAiRight = {x,y,z};
			self.setWorkingWidth = VarioSpreadingWidth.setWorkingWidth;
		else
			self.isSpreader = false;
			print("self.sprayUsageScale == nil -> VarioSpreadingWidth can not be used!")
		end;
	end;
end;

function VarioSpreadingWidth:postLoad(savegame)
	if savegame ~= nil and not savegame.resetVehicles and self.isSpreader and self.varioSpreadingWidth ~= nil then
		local workingWidth = Utils.getNoNil(getXMLInt(savegame.xmlFile, savegame.key .. "#workingWidth"), self.varioSpreadingWidth.workingWidth);
		if workingWidth ~= self.varioSpreadingWidth.workingWidth then
			self:setWorkingWidth(workingWidth, false);
		end;
	end;
end;

function VarioSpreadingWidth:getSaveAttributesAndNodes(nodeIdent)
	local attributes = "";
	if self.isSpreader and self.varioSpreadingWidth ~= nil then
			attributes = 'workingWidth="' .. tostring(self.varioSpreadingWidth.workingWidth) ..'"';
	end;
	return attributes, nil;
end;

function VarioSpreadingWidth:readStream(streamId, connection)
	if self.isSpreader and self.varioSpreadingWidth ~= nil then
		local workingWidth = streamReadInt8(streamId);
		self:setWorkingWidth(workingWidth, true);
	end;
end;

function VarioSpreadingWidth:writeStream(streamId, connection)
	if self.isSpreader and self.varioSpreadingWidth ~= nil then
		streamWriteInt8(streamId, self.varioSpreadingWidth.workingWidth);
	end;
end;

function VarioSpreadingWidth:delete()
end;

function VarioSpreadingWidth:mouseEvent(posX, posY, isDown, isUp, button)
end;

function VarioSpreadingWidth:keyEvent(unicode, sym, modifier, isDown)
end;

function VarioSpreadingWidth:setWorkingWidth(workingWidth, noEventSend)
	SetWorkingWidthEvent.sendEvent(self, workingWidth, noEventSend);
	self.varioSpreadingWidth.workingWidth = workingWidth;
	local orgWorkWidth;
	local orgXw,orgYw,orgZw;
	local orgXh,orgYh,orgZh;
	for k,v in pairs(self.orgWorkAreas) do
		orgWorkWidth = v.workWidth
		orgXw = v.width.x;
		orgYw = v.width.y;
		orgZw = v.width.z;
		orgXh = v.height.x;
		orgYh = v.height.y;
		orgZh = v.height.z;
	end;
	local xFactor = workingWidth/orgWorkWidth;
	for k,v in pairs(self.workAreas) do
		local zFactor = xFactor * .8;
		if workingWidth <= orgWorkWidth then
			zFactor = 1;
		end;
		local xW,yW,zW = orgXw,orgYw,orgZw;
		xW = xW*xFactor;
		zW = zW*zFactor;
		setTranslation(v.width,xW,yW,zW)
		local xH,yH,zH = orgXh,orgYh,orgZh;
		xH = xH*xFactor;
		zH = zH*zFactor;
		setTranslation(v.height,xH,yH,zH)
	end;
	local x,y,z = unpack(self.orgAiLeft);
	setTranslation(self.aiLeftMarker,x*xFactor,y,z);
	x,y,z = unpack(self.orgAiRight);
	setTranslation(self.aiRightMarker,x*xFactor,y,z);
	if self.sprayerEffects ~= nil then
		for _,sprayerEffect in pairs(self.sprayerEffects) do
			if sprayerEffect.effects ~= nil then
				for _,effect in pairs(sprayerEffect.effects) do
					local xFactor = workingWidth/orgWorkWidth;
					local zFactor = xFactor * .8;
					if workingWidth <= orgWorkWidth then
						zFactor = 1;
					end;
					setScale(effect.node,xFactor,1,zFactor);
				end;
			end;
		end;
	end;
	if self.sprayParticleSystems ~= nil then
		for _,sprayParticleSystem in pairs(self.sprayParticleSystems) do
			for _,sprayParticle in pairs(sprayParticleSystem) do
				local xFactor = workingWidth/orgWorkWidth;
				local zFactor = xFactor * .8;
				if workingWidth <= orgWorkWidth then
					zFactor = 1;
				end;
				setScale(sprayParticle.emitterShape,xFactor,1,zFactor);
			end;
		end;
	end;
	if self.sprayUsageScale ~= nil then
		self.sprayUsageScale.workingWidth = self.varioSpreadingWidth.workingWidth;
	end;
end;

function VarioSpreadingWidth:update(dt)
	if self:getIsActive() and self.isSpreader then
	  if self.isClient then
			if self:getIsActiveForInput(false) and not self:hasInputConflictWithSelection() then
				if InputBinding.isPressed(InputBinding.SPREADER_WORKINGWIDTH_BUTTON) then
					self.varioSpreadingWidth.buttonIsPressed = true;
					if InputBinding.hasEvent(InputBinding.SPREADER_WORKINGWIDTH_UP) and self.varioSpreadingWidth.workingWidth < self.varioSpreadingWidth.maxWorkWidth then
						self.varioSpreadingWidth.buttonTimer = g_currentMission.time + 200;
						self.varioSpreadingWidth.workingWidth = self.varioSpreadingWidth.workingWidth + 1;
					elseif InputBinding.hasEvent(InputBinding.SPREADER_WORKINGWIDTH_DOWN) and self.varioSpreadingWidth.workingWidth > self.varioSpreadingWidth.minWorkWidth then
						self.varioSpreadingWidth.buttonTimer = g_currentMission.time + 200;
						self.varioSpreadingWidth.workingWidth = self.varioSpreadingWidth.workingWidth - 1;
					end;
					if InputBinding.isPressed(InputBinding.SPREADER_WORKINGWIDTH_UP) and self.varioSpreadingWidth.workingWidth < self.varioSpreadingWidth.maxWorkWidth then
						if self.varioSpreadingWidth.buttonTimer < g_currentMission.time then
							self.varioSpreadingWidth.buttonTimer = g_currentMission.time + 100;
							self.varioSpreadingWidth.workingWidth = self.varioSpreadingWidth.workingWidth + 1;
						end;
					elseif InputBinding.isPressed(InputBinding.SPREADER_WORKINGWIDTH_DOWN) and self.varioSpreadingWidth.workingWidth > self.varioSpreadingWidth.minWorkWidth then
						if self.varioSpreadingWidth.buttonTimer < g_currentMission.time then
							self.varioSpreadingWidth.buttonTimer = g_currentMission.time + 100;
							self.varioSpreadingWidth.workingWidth = self.varioSpreadingWidth.workingWidth - 1;
						end;
					end;
				else
					if self.varioSpreadingWidth.buttonIsPressed then
						self.varioSpreadingWidth.buttonIsPressed = false;
						if self.varioSpreadingWidth.workingWidth ~= self.varioSpreadingWidth.workingWidthBackup then
							self:setWorkingWidth(self.varioSpreadingWidth.workingWidth);
							self.varioSpreadingWidth.workingWidthBackup = self.varioSpreadingWidth.workingWidth;
						end;
					end;
				end;
			end;
		end;
	end;
end;

function VarioSpreadingWidth:updateTick(dt)
end;

function VarioSpreadingWidth:draw()
	if self.isClient and self.isSpreader then
		if self.varioSpreadingWidth ~= nil then
			if self.varioSpreadingWidth.buttonIsPressed then
				g_currentMission:addHelpButtonText(g_i18n:getText("action_increaseWorkingWidth"), InputBinding.SPREADER_WORKINGWIDTH_UP, nil, GS_PRIO_NORMAL);
				g_currentMission:addHelpButtonText(g_i18n:getText("action_reduceWorkingWidth"), InputBinding.SPREADER_WORKINGWIDTH_DOWN, nil, GS_PRIO_NORMAL);
			else
				g_currentMission:addHelpButtonText(g_i18n:getText("action_adjustWorkingWidth"), InputBinding.SPREADER_WORKINGWIDTH_BUTTON, nil, GS_PRIO_NORMAL);
			end;
			g_currentMission:addExtraPrintText(g_i18n:getText("info_spreaderWorkingWidth").."  "..tostring(self.varioSpreadingWidth.workingWidth).."m");
		end;
	end;
end;


------------------------------------------------------------------------------------
------------------------------------------------------------------------------------
SetWorkingWidthEvent = {};
SetWorkingWidthEvent_mt = Class(SetWorkingWidthEvent, Event);

InitEventClass(SetWorkingWidthEvent, "SetWorkingWidthEvent");

function SetWorkingWidthEvent:emptyNew()
  local self = Event:new(SetWorkingWidthEvent_mt);
  return self;
end;

function SetWorkingWidthEvent:new(vehicle, workingWidth)
  local self = SetWorkingWidthEvent:emptyNew()
  self.vehicle = vehicle;
	self.workingWidth = workingWidth;
  return self;
end;

function SetWorkingWidthEvent:readStream(streamId, connection)
	self.vehicle = readNetworkNodeObject(streamId);
	self.workingWidth = streamReadInt8(streamId);
	self:run(connection);
end;

function SetWorkingWidthEvent:writeStream(streamId, connection)
  writeNetworkNodeObject(streamId, self.vehicle);
	streamWriteInt8(streamId, self.workingWidth);
end;

function SetWorkingWidthEvent:run(connection)
	self.vehicle:setWorkingWidth(self.workingWidth, true);
	if not connection:getIsServer() then
		g_server:broadcastEvent(SetWorkingWidthEvent:new(self.vehicle, self.workingWidth), nil, connection, self.vehicle);
	end;
end;

function SetWorkingWidthEvent.sendEvent(vehicle, workingWidth, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(SetWorkingWidthEvent:new(vehicle, workingWidth), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(SetWorkingWidthEvent:new(vehicle, workingWidth));
		end;
	end;
end;